Package: pcaPP
Version: 2.0-5
VersionNote: Released 2.0-4-1 on 2024-08-09 on CRAN
Title: Robust PCA by Projection Pursuit
Maintainer: Valentin Todorov <valentin.todorov@chello.at>
Authors@R: c(person(given = "Peter", family = "Filzmoser", role = "aut"),
              person(given = "Heinrich", family = "Fritz", role = "aut"),
              person(given = "Klaudius", family = "Kalcher", role = "aut"),
              person(given = "Valentin", family = "Todorov", role = "cre",
                     email = "valentin.todorov@chello.at"))
Depends: R(>= 3.6.2)
Imports: mvtnorm, methods
Suggests: robustbase
Description: Provides functions for robust PCA by projection pursuit.
    The methods are described in Croux et al. (2006) <doi:10.2139/ssrn.968376>,
    Croux et al. (2013) <doi:10.1080/00401706.2012.727746>,
    Todorov and Filzmoser (2013) <doi:10.1007/978-3-642-33042-1_31>.
License: GPL (>= 3)
URL: https://github.com/valentint/pcaPP
BugReports: https://github.com/valentint/pcaPP/issues
NeedsCompilation: yes
Packaged: 2024-08-19 10:48:56 UTC; valen
Author: Peter Filzmoser [aut],
  Heinrich Fritz [aut],
  Klaudius Kalcher [aut],
  Valentin Todorov [cre]
Repository: CRAN
Date/Publication: 2024-08-19 13:00:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 15:51:14 UTC; unix
Archs: pcaPP.so.dSYM
