
<!-- README.md is generated from README.Rmd. Please edit that file -->

# `pcaPP`: Robust PCA by Projection Pursuit

<!-- badges: start -->

[![CRAN
version](https://www.r-pkg.org/badges/version/pcaPP)](https://cran.r-project.org/package=pcaPP)
[![R-CMD-check](https://github.com/valentint/pcaPP/workflows/R-CMD-check/badge.svg)](https://github.com/valentint/pcaPP/actions)
[![downloads](https://cranlogs.r-pkg.org/badges/pcaPP)](https://cran.r-project.org/package=pcaPP)
[![downloads](https://cranlogs.r-pkg.org/badges/grand-total/pcaPP)](https://cran.r-project.org/package=pcaPP)
[![license](https://img.shields.io/badge/license-GPL--3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0.en.html)
<!-- badges: end -->

## Installation

The `pcaPP` package is on CRAN (The Comprehensive R Archive Network) and
the latest release can be easily installed using the command

    install.packages("pcaPP")

## Building from source

To install the latest stable development version from GitHub, you can
pull this repository and install it using

    ## install.packages("remotes")
    remotes::install_github("valentint/pcaPP")

Of course, if you have already installed `remotes`, you can skip the
first line (I have commented it out).

## Community guidelines

### Report issues and request features

If you experience any bugs or issues or if you have any suggestions for
additional features, please submit an issue via the
[*Issues*](https://github.com/valentint/pcaPP/issues) tab of this
repository. Please have a look at existing issues first to see if your
problem or feature request has already been discussed.

### Contribute to the package

If you want to contribute to the package, you can fork this repository
and create a pull request after implementing the desired functionality.

### Ask for help

If you need help using the package, or if you are interested in
collaborations related to this project, please get in touch with the
package maintainer.
